package de.uni_passau.fim.seibt.gitwrapper.process;

import java.io.File;
import java.io.IOException;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.io.IOUtils;

/**
 * Contains static methods for executing commands and gathering their output and exit code.
 */
public class ProcessExecutor {

    private static final Logger LOG = Logger.getLogger(ProcessExecutor.class.getCanonicalName());

    /**
     * The result of a process execution.
     */
    public static class ExecRes {
        public int exitCode;
        public String output;
    }

    /**
     * Executes the given <code>cmd</code> with the supplied <code>parameters</code>.
     *
     * @param cmd
     *         the command to execute
     * @param workingDir
     *         the working directory for the execution
     * @param redirectError
     *         whether to merge the standard output and error streams
     * @param parameters
     *         the parameters for the command (may be null or none)
     * @return the result of the execution or an empty {@link Optional} if there is an exception executing the command
     */
    public static Optional<ExecRes> exec(String cmd, File workingDir, boolean redirectError, String... parameters) {

        if (parameters == null) {
            return exec(workingDir, redirectError, cmd);
        }

        String[] cmdArray = new String[parameters.length + 1];

        cmdArray[0] = cmd;
        System.arraycopy(parameters, 0, cmdArray, 1, parameters.length);

        return exec(workingDir, redirectError, cmdArray);
    }

    /**
     * Executes the given <code>command</code>. If <code>command</code> is <code>null</code> no var-args are given
     * and empty {@link Optional} will be returned.
     *
     * @param workingDir
     *         the working directory for the execution
     * @param redirectError
     *         whether to merge the standard output and error streams
     * @param command
     *         the command to be executed
     * @return the result of the execution or an empty {@link Optional} if there is an exception executing the command
     */
    public static Optional<ExecRes> exec(File workingDir, boolean redirectError, String... command) {

        if (command == null) {
            return Optional.empty();
        }

        ProcessBuilder builder = new ProcessBuilder();

        builder.command(command);
        builder.directory(workingDir);
        builder.redirectErrorStream(redirectError);

        return exec(builder);
    }

    /**
     * Executes the command represented by the given <code>builder</code>.
     *
     * @param builder
     *         the {@link ProcessBuilder} to be executed
     * @return the result of the execution or an empty {@link Optional} if there is an exception executing the command
     */
    public static Optional<ExecRes> exec(ProcessBuilder builder) {
        String cmd = String.join(" ", builder.command());
        LOG.fine(() -> String.format("Executing '%s'.", cmd));

        try {
            ExecRes res = new ExecRes();
            Process p = builder.start();

            res.output = IOUtils.toString(p.getInputStream());
            res.exitCode = p.waitFor();

            LOG.fine(() -> String.format("Execution of '%s' returned exit code %d.", cmd, res.exitCode));
            LOG.finest(() -> String.format("Execution of '%s' output:%n%s", cmd, res.output));

            return Optional.of(res);
        } catch (IOException | InterruptedException e) {
            LOG.log(Level.SEVERE, e, () -> String.format("Exception executing '%s'.", cmd));
            return Optional.empty();
        }
    }
}
